###############################################################################
#Software for calibrating hydrological models by using the Whittle's likelihood
###############################################################################

#This software performs the estimation of a hydrological model by using the
#Whittle's likelihood. Calibration is performed by matching contemporary observed and
#simulated data.
#Please contact me at alberto.montanari@unibo.it if you find problems in using
#the software. You may test it by using the test files I am providing with this package.
#Please note that:
#1) you need genoud installed. You can download it at 
#http://cran.at.r-project.org/src/contrib/Descriptions/rgenoud.html
#2) you need to put the observed values in a vector in R which is named "osser".
#3) you need to put in the R main directory an executable file named "model.exe" which
#is your hydrological model to be calibrated. It should read the parameters to calibrate
#in a text file named "admold.par". It has to write the simulated output data in text a file 
#named "adm.ris", in just one column. Simulated and observed data must have the same length.

#Please note also that:
#1) if you need to calibrate the model, you will have to use the funcyion calib. As arguments,
#it takes the number nvars of parameters and a matrix of dimension (nvars,2), which contains
#upper and lower limits of the parameter values.
#2) if you need to only compute the likelihood of a parameter set, you will have to use the 
#function whit, which takes as argument the model parameter vector.
#3) The model residual error is autoregressive of order one.

#Please report any bug to me. I would appreciate to keep track of those who are using
#the siftware, so I can send updates to the, Therefore, please drop me an e-mail when you
#download this file.


calib=function(nvars,limits)
{
assign("osser",value=osser)
pr1=genoud(whit,nvars=nvars,max=FALSE,Domains=limits,solution.tolerance=11,gradient.check=FALSE,wait.generations=3,boundary.enforcement=2)
return(pr1)
};

###############################################
#Likelihood function
###############################################

whit=function(params)
{
assign("osser",value=osser)
write(params,file="admold.par",ncolumns=1)
system("model.exe", show.output.on.console = TRUE)
pr1=scan("adm.ris",quiet=TRUE)
err=pr1-osser
pr20=ar(err,order.max=1)
pr21=pr20$ar[1]
pr22=var(pr20$resid[is.na(pr20$resid)==F])
toll=mean(osser)/5
pr10=whitdef(pr1,osser,pr21,pr22,toll)
return(pr10$B)
};


###############################################
#Computation of likelihood
###############################################


whitdef=function(xsim,xobs,arcoef,varerr,toll)
{
	nmax=length(xsim)
	nmax1=length(xobs)
	if(nmax!=nmax1)
	{
	cat("Observed and simulated series have not equal length", fill=T)
	cat("Procedure interrupted", fill=T)	
	return()
	}
 	 
#standardize data

	varsim=var(xsim)
	varobs=var(xobs)
	nhalfm=trunc((nmax-1)/2)

#computation of the bigger integer closest to n/2.

#periodogram of the data

xsimper=per(xsim)[2:(nhalfm+1)]
xobsper=per(xobs)[2:(nhalfm+1)]


#Smoothing of the simulated periodogram is much stronger has it is at the denominator in the computation below.
 
    p=1
    
#Autoregressive order of error model    

#   cat("in fspecARIMA",fill=T)

 	phi=c()
 	psi=c()
 
 	x=1:nhalfm
 	x=2*pi/nmax*x
 
#calculation of f at fourier frequencies
 
 	far=(1:nhalfm)/(1:nhalfm)

#Inizialization of periodogram of autoregressive part - error model

 	if(p>0)
 	 {phi=arcoef
 	 cosar=cos(cbind(x)%*%rbind(1:p))
 	 sinar=sin(cbind(x)%*%rbind(1:p))
 	 Rar=cosar%*%phi
 	 Iar=sinar%*%phi
 	 far=(1-Rar)**2+Iar**2
 	 }
 	fspec=1/far
	fspec=fspec*varerr/2/pi

#Periodogram of the error model calculated

#Replication of the periodogram of the simulated series

	xsimper=xsimper+fspec

#Computation of the periodogram of rainfall-runoff model plus error model

#Find likelihood

	yf=xobsper/xsimper

#Step of likelihood computation of the rainfall-runoff model plus error model. Note that xsimper is at the denominator.


B=(2/nmax)*(sum(yf[xsimper>0])+sum(log(xsimper[xsimper>0])))

#Computation of likelihood of the rainfall-runoff model plus error model. Note that 
#negative ordinates of simulated periodogram are excluded

B3=abs(mean(xsim)-mean(xobs))

#Computation of the difference between mean values of simulated and observed series. Only models that
#give equality of these mean values can be accepted.

if(B3>toll)
{
B=B*100
}

#Artefact: the objective function is penalised if the means are different.


result=list(B=B)
return(result)
};

###############################################
#definition of the periodogram
###############################################

per=function(z)
	{n=length(z)
	(Mod(fft(z))**2/(2*pi*n)) [1:(n%/%2+1)]
	};
